function [Theta,J,sigma_e,cov,err,Yprev] = MinQuad3( Yvet , Nseq , Ord , graph )
% [ Theta , J , sigma_e , cov , err , Yprev] =
%                   MinQuad3( Yvet , Nseq , Ord , graph ) restituisce il
% vettore dei parametri Theta , soluzione dei minimi quadrati e l'errore
% quadratico medio di previsione , J , sugli Nseq dati disponibili ,conte=
% nuti in Yvet , matrice di campioni dell'ingresso e di uscita , divisi
% in nc colonne , per un modello di ordine Ord; cov e' la covarianza della
% stima , mentre sigma_e , la varianza del  processo  stocastico e(.).
% Nella prima colonna di Yvet compare l'uscita del processo e di seguito
% le nc-1 colonne degli ingressi.
% Il vettore dei parametri avra' dimensione  nc*Ord,  ordinato secondo
% [An,...,A1,Bn1,...B11,Bnr,...B1r] se n e' l'orine Ord e r gli ingressi.
% Stampa una simulazione se graph == 1.
% Restituisce l'errore di previsione.

  [ nr , nc ] = size( Yvet );    % ci sono ingressi ed uscite;

  if( nc >= 2 ) , %  Prime colonne ingressi e ultima col. uscita.

  H=[ myhank(Yvet(:,nc),Nseq+Ord,Ord,0), myhank(Yvet(:,1),Nseq+Ord , Ord ,0 )];

  else % Nessun ingresso, solo una uscita. 
  
  H= myhank( Yvet(:,1) , Nseq+Ord , Ord , 0 ); % Yvet e' un vettore.

  end;

  if( nc > 2 ),

    for input = 2:nc-1,
     
    H = [ H , myhank( Yvet(:,input) , Nseq+Ord , Ord , 0 )];

    end;

  end;
   
     Theta = pinv( H ) * Yvet( Ord+1:Nseq+Ord , nc) ;
     
   %  Theta = ( inv( H'* H ) )*( H')*Yvet( Ord+1:Nseq+Ord , nc) ;

if (graph == 1),


figure
plot([1:Nseq],H*Theta,'-',[1:Nseq],Yvet( Ord+1:Nseq+Ord ,nc ),'--');
title('Simulazione linea cont. e uscita misurata linea tratt.')
ylabel('Y (-) simulata e (--) misurata'),xlabel('Campioni'), pause;

%print -dps simPe.ps

end;

  
  Yprev = H*Theta;
  errps = Yvet( Ord+1:Nseq+Ord ,nc )-Yprev;

 

if (graph == 1),

figure
plot([1:Nseq],errps,'-');
title('Errore di previsione')
ylabel('Yv-Yp'),xlabel('Campioni'), pause;

%print -dps errPe.ps

end;


   J = ( errps'* errps )/Nseq;

   sigma_e = ( J * Nseq )/( Nseq - nc*Ord );

   cov = sigma_e * pinv( H' * H );

   err = errps;


return



